#include "meta.h"
#include <math.h> //for atan() in DoAI() and in the graphics shadows (SLOW! FIX! :| )

/* note: the origin is the top-left, x inreases right, and y increases down; 0 rotation points up,
	angles increse rotating to the left, and a full circle is 256 degrees (this stuff wasn't exactly
	planned, it just turned out that way) */

void InitGame(void) {
	int i;

	/* get the frequency of the performance counter */
	if(!QueryPerformanceFrequency(&timerFrequency) || !timerFrequency.QuadPart) die("find a system performance timer");

	/* start recording the time (now will be then when the program starts :]) */
	QueryPerformanceCounter(&then);

	/* game time starts at zero */
	secNow = 0;

	srand(then.QuadPart); //seed the random number generator

	//initialize all ships
	for(i = 0; i < NUMSHIPS; i++) {
		ship[i].p.x = rand() % SCRX;
		ship[i].p.y = rand() % SCRY;
		ship[i].r = rand() & 255; /* same as % 256, but faster */
		ship[i].inertia.x = 0;
		ship[i].inertia.y = 0;
		ship[i].shield = MAXSHIELDS;
		ship[i].whenshoot = 0;
		ship[i].whenshield = 0;
	}

	//set the star in the middle of the screen
	light[0].lux = STAR_LUX;
	light[0].p.x = HLFX;
	light[0].p.y = HLFY;
	numlights = 1;

	//initialize shots on screen
	numshots = 0;
}

int GameMain(BOOL multiplayer) {
	QueryPerformanceCounter(&now); //get the current time in arbitrary system query units
	dtime = (float)(now.QuadPart - then.QuadPart) / timerFrequency.QuadPart; //calculate the time difference in seconds as a float
	/* if(dtime > MAX_FRAME_LAG) dtime = 0; don't lag, e.g. when task is deactivated (this messes up time-delayed events, so don't use) */
	secNow = now.QuadPart / timerFrequency.QuadPart;
	then = now; //for next time
	DoPhysics(); //move the stuff on screen around, collision detection, etc
	CheckValues(); //wrap to screen, curb speed and shields, move associated lights, recharge shields, etc.
	DoGraphics(); //display the graphics
	DoInput(); //check keyboard input and respond appropreately
	if(multiplayer) DoInput2(); //check the second player
	else DoAI(); //move the computer
	if(ship[0].shield <= 0) return 1;
	else if(ship[1].shield <= 0) return 0;
	else return -1;
}

void AddShot(int shipindex) {
	if(secNow >= ship[shipindex].whenshoot && numshots < MAXSHOTS - 1) { //the weap has recharged and there aren't too many shots on screen
		//set time when wep will be recharged
		ship[shipindex].whenshoot = secNow + WEPRECHARGE;
		//set shot pos WEPSTARTDIST in front of ship
		shot[numshots].p.x = ship[shipindex].p.x - Sin[ship[shipindex].r] * WEPSTARTDIST;
		shot[numshots].p.y = ship[shipindex].p.y - Cos[ship[shipindex].r] * WEPSTARTDIST;
		//add WEPSPEED to the speed of the ship
		shot[numshots].inertia.x = ship[shipindex].inertia.x - (WEPSPEED * Sin[ship[shipindex].r]);
		shot[numshots].inertia.y = ship[shipindex].inertia.y - (WEPSPEED * Cos[ship[shipindex].r]);
		//set the shot to expire after WEPRANGE
		shot[numshots].whenexpires = secNow + WEPRANGE;
		//the light associated w/ the shot is the next free one
		shot[numshots].lightindex = numlights;
		//increment the shot counter
		numshots++;
		//set the light values to match
		light[numlights].p.x = shot[numshots].p.x;
		light[numlights].p.y = shot[numshots].p.y;
		light[numlights].lux = WEP_LUX;
		numlights++;
	}
}

void KillShot(int shotindex) {
	int i;

	KillLight(shot[shotindex].lightindex); //get rid of the associated light
	for(i = 0; i < numshots; i++) {
		//since the light associated with the shot has been deleted, adjust the other assaciations to point to the new light numbers
		if(shot[i].lightindex >= shot[shotindex].lightindex) shot[i].lightindex--;
	}
	//shift the following shots down to fill shotindex's position
	for(i = shotindex; i < numshots; i++) {
		shot[i].lightindex = shot[i + 1].lightindex;
		shot[i].p.x = shot[i + 1].p.x;
		shot[i].p.y = shot[i + 1].p.y;
		shot[i].inertia.x = shot[i + 1].inertia.x;
		shot[i].inertia.y = shot[i + 1].inertia.y;
		shot[i].whenexpires = shot[i + 1].whenexpires;
	}
	numshots--; //reduce the counter of the number of shots
}

void KillLight(int lightindex) {
	int i;

	if(numlights > 1) { /* I've given up trying to figue out why the sun just keeps disappering, so I'll force it not to */
		numlights--; //reduce the counter of the number of lights
		//shift the following lights down to fill lightindex's position
		for(i = lightindex; i < numlights; i++) {
			light[i].p.y = light[i + 1].p.y;
			light[i].lux = light[i + 1].lux;
			light[i].p.x = light[i + 1].p.x;
		}
	}
}

void CheckValues(void) {
	int i;

	for(i = 0; i < NUMSHIPS; i++) {
		//wrap pos horizontally
		if(ship[i].p.x < LBOUND) {
//			ship[i].p.x += HZONE;
			ship[i].p.x = LBOUND;
			ship[i].inertia.x = -ship[i].inertia.x * BOUNCERATIO;
		}
		else if(ship[i].p.x >= RBOUND) {
//			ship[i].p.x -= HZONE;
			ship[i].p.x = RBOUND - 1;
			ship[i].inertia.x = -ship[i].inertia.x * BOUNCERATIO;
		}
		//wrap pos vertically
		if(ship[i].p.y < TBOUND) {
//			ship[i].p.y += VZONE;
			ship[i].p.y = TBOUND;
			ship[i].inertia.y = -ship[i].inertia.y * BOUNCERATIO;
		}
		else if(ship[i].p.y >= BBOUND) {
//			ship[i].p.y -= VZONE;
			ship[i].p.y = BBOUND - 1;
			ship[i].inertia.y = -ship[i].inertia.y * BOUNCERATIO;
		}
		{
			float speed = hypot(ship[i].inertia.x, ship[i].inertia.y);

			if(speed > MAXSPEED) {
				ship[i].inertia.x = (ship[i].inertia.x / speed) * MAXSPEED;
				ship[i].inertia.y = (ship[i].inertia.y / speed) * MAXSPEED;
			}
			if(speed < MINSPEEDTOLERANCE) { //the ship is going slow enough to just stop (it's annoying always moving a tiny bit)
				ship[i].inertia.x = 0;
				ship[i].inertia.y = 0;
			}
		}
		//recharge shields
		if(secNow > ship[i].whenshield) {
			ship[i].shield++;
			ship[i].whenshield = secNow + SHIELDRECHARGE;
		}
		//check too high shield values
		if(ship[i].shield > MAXSHIELDS) ship[i].shield = MAXSHIELDS;
		//check too low shield values
		if(ship[i].shield <= 0) {
			//FIXME: die
			//DestroyWindow(hWnd);
		}
	}
	for(i = 0; i < numshots; i++) {
		//wrap pos horizontally
		/* now bounces, instead */
		if(shot[i].p.x < LBOUND) {
//			shot[i].p.x += HZONE;
			shot[i].p.x = LBOUND;
			shot[i].inertia.x = -shot[i].inertia.x * BOUNCERATIO;
		}
		else if(shot[i].p.x >= RBOUND) {
//			shot[i].p.x -= HZONE;
			shot[i].p.x = RBOUND - 1;
			shot[i].inertia.x = -shot[i].inertia.x * BOUNCERATIO;
		}
		//wrap pos vertically
		/* now bounces, instead */
		if(shot[i].p.y < TBOUND) {
//			shot[i].p.y += VZONE;
			shot[i].p.y = TBOUND;
			shot[i].inertia.y = -shot[i].inertia.y * BOUNCERATIO;
		}
		else if(shot[i].p.y >= BBOUND) {
//			shot[i].p.y -= VZONE;
			shot[i].p.y = BBOUND - 1;
			shot[i].inertia.y = -shot[i].inertia.y * BOUNCERATIO;
		}
		{
			float speed = hypot(shot[i].inertia.x, shot[i].inertia.y);

			if(speed > MAXWEPSPEED) {
				shot[i].inertia.x = (shot[i].inertia.x / speed) * MAXWEPSPEED;
				shot[i].inertia.y = (shot[i].inertia.y / speed) * MAXWEPSPEED;
			}
		}
		//make the light follow the shot
		light[shot[i].lightindex].p.x = shot[i].p.x;
		light[shot[i].lightindex].p.y = shot[i].p.y;
		//check if expired, if so, delete
		if(shot[i].whenexpires <= secNow) {
			KillShot(i);
		}
	}
	//for(i = 0; i < numlights; i++) {
		//nothing to check for the lights yet
	//}
}

void DoPhysics(void) {
	int i, j;

	//move the ships
	for(i = 0; i < NUMSHIPS; i++) {
		ship[i].p.x += ship[i].inertia.x * dtime;
		ship[i].p.y += ship[i].inertia.y * dtime;
	}

	//move the weps
	for(i = 0; i < numshots; i++) {
		shot[i].p.x += shot[i].inertia.x * dtime;
		shot[i].p.y += shot[i].inertia.y * dtime;
	}
	//do collision detection with shots on screen
	for(i = 0; i < numshots; i++) {
		for(j = 0; j < NUMSHIPS; j++) {
			//note: check further if the part of the ship hit is opaque? (takes time when rotating manually, but later . . . )
			// also improve this by checking the entire path (some vector function) and not just the end
			if(shot[i].p.x >= ship[j].p.x - HLFSZE && shot[i].p.x <= ship[j].p.x + HLFSZE && shot[i].p.y >= ship[j].p.y - HLFSZE && shot[i].p.y <= ship[j].p.y + HLFSZE) {
				ship[j].shield -= WEPDAMAGE; //damage the ship's shields
				ship[j].inertia.x += shot[i].inertia.x * SHOT_IMPACT; //push the ship around
				ship[j].inertia.y += shot[i].inertia.y * SHOT_IMPACT; //note: make a doexplosion func to just push everything around out and seed an explosion graphic
				KillShot(i); //destroy the shot
			}
		}
	}
}

void DoGraphics(void) {
	static const byte lght[SPRSZE][SPRSZE] = {	10,10,10,9,9,9,8,8,8,8,8,9,9,10,10,10,//A circular pattern
												10,9,9,9,8,8,7,7,7,7,8,8,9,9,9,10,
												10,9,8,8,7,7,6,6,6,6,7,7,8,8,9,10,
												9,9,8,7,6,6,5,5,5,5,6,6,7,8,9,9,
												9,8,7,6,5,5,4,4,4,4,5,5,6,7,8,9,
												9,8,7,6,5,4,4,3,3,4,4,5,6,7,8,9,
												8,7,6,5,4,4,3,2,2,3,4,4,5,6,7,8,
												8,7,6,5,4,3,2,1,1,2,3,4,5,6,7,8,
												8,7,6,5,4,3,2,1,1,2,3,4,5,6,7,8,
												8,7,6,5,4,4,3,2,2,3,4,4,5,6,7,8,
												9,8,7,6,5,4,4,3,3,4,4,5,6,7,8,9,
												9,8,7,6,5,5,4,4,4,4,5,5,6,7,8,9,
												9,9,8,7,6,6,5,5,5,5,6,6,7,8,9,9,
												10,9,8,8,7,7,6,6,6,6,7,7,8,8,9,10,
												10,9,9,9,8,8,7,7,7,7,8,8,9,9,9,10,
												10,10,10,9,9,9,8,8,8,8,9,9,9,10,10,10	};
	static const byte angs[SPRSZE][SPRSZE] = {	96,92,88,84,80,76,70,68,60,58,52,48,44,40,36,32,//A radial pattern
												100,96,92,84,80,76,70,68,60,58,52,48,44,36,32,28,
												104,100,96,92,84,80,72,68,60,56,48,44,36,32,28,24,
												108,108,100,96,88,80,72,68,60,56,48,40,32,28,20,20,
												112,112,108,104,96,88,80,68,60,48,40,32,24,20,16,16,
												116,116,112,112,104,96,80,68,60,48,32,24,16,16,12,12,
												122,122,120,120,112,112,96,68,60,32,16,16,8,8,6,6,
												124,124,124,124,124,124,124,96,32,4,4,4,4,4,4,4,
												132,132,132,132,132,132,132,160,224,252,252,252,252,252,252,252,
												134,134,136,136,144,144,160,188,196,224,240,240,248,248,250,250,
												140,140,144,144,152,160,176,188,196,208,224,232,240,240,244,244,
												144,144,148,152,160,168,176,188,196,208,216,224,232,236,240,240,
												148,148,156,160,168,176,184,188,196,200,208,216,224,218,236,236,
												152,156,160,164,172,176,184,188,196,200,208,212,220,224,218,232,
												156,160,164,172,176,180,186,188,196,198,204,208,212,220,224,218,
												160,164,168,172,176,180,186,188,196,198,204,208,212,216,220,224	};
	static const byte sprt[2][SPRSZE][SPRSZE] = {00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,//A ship graphic
												00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,
												00,00,00,00,00,00,00,255,00,00,00,00,00,00,00,00,
												00,00,00,00,00,00,00,70,70,00,00,00,00,00,00,00,
												00,00,00,00,00,00,70,90,90,70,00,00,00,00,00,00,
												00,00,00,30,90,90,90,90,90,90,90,30,60,90,60,00,
												30,90,90,90,90,90,90,90,90,90,90,90,90,90,90,60,
												90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,50,
												90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,50,
												30,90,90,90,90,90,90,90,90,90,90,90,90,90,90,60,
												00,00,00,30,90,90,90,90,90,90,90,30,60,90,60,00,
												00,00,00,00,00,00,70,90,90,70,00,00,00,00,00,00,
												00,00,00,00,00,00,00,70,70,00,00,00,00,00,00,00,
												00,00,00,00,00,00,00,255,00,00,00,00,00,00,00,00,
												00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,
												00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,//}
												00,00,00,60,90,90,90,90,60,00,00,60,90,30,00,00,//Second ship graphic
												00,00,70,90,90,90,90,50,90,90,90,90,90,90,30,00,
												00,30,90,90,00,70,90,00,30,90,70,00,00,60,90,00,
												00,90,90,30,00,00,70,00,00,70,00,00,00,00,90,70,
												00,90,90,90,60,00,70,00,00,70,00,00,00,00,90,90,
												00,00,00,00,00,00,90,20,20,90,90,00,00,00,90,90,
												00,60,90,90,60,30,90,90,90,90,90,90,70,70,90,60,
												70,90,90,90,90,90,90,90,90,90,90,90,90,90,70,30,
												70,90,90,90,90,90,90,90,90,90,90,90,90,90,70,30,
												00,60,90,90,60,30,90,90,90,90,90,90,70,70,90,60,
												00,00,00,00,00,00,90,20,20,90,90,00,00,00,90,90,
												00,90,90,90,60,00,70,00,00,70,00,00,00,00,90,90,
												00,90,90,30,00,00,70,00,00,70,00,00,00,00,90,70,
												00,30,90,90,00,70,90,00,30,90,70,00,00,60,90,00,
												00,00,70,90,90,90,90,50,90,90,90,90,90,90,30,00,
												00,00,00,60,90,90,90,90,60,00,00,60,90,30,00,00	};
	vec2_it pxl, sprpos, centered, sprpxl, dtolight;
	int disttolight[MAXLIGHTS], i, j, tmpclr;
	BOOL lightaffectsship[MAXLIGHTS];
	byte c, lightang[MAXLIGHTS], angfromlight;

	//draw the lights (should add some sort of halo)
	for(i = 0; i < numlights; i++) {
		pxl.x = light[i].p.x;
		pxl.y = light[i].p.y;
		c = SCRMAXCOLOR;
		pset(pxl.x, pxl.y, c);
	}

	//draw the shots
	for(i = 0; i < numshots; i++) {
		pxl.x = shot[i].p.x;
		pxl.y = shot[i].p.y;
		c = SHOT_COLOR;
		pset(pxl.x, pxl.y, c);
	}

	//draw the ships
	for(i = 0; i < NUMSHIPS; i++) {
		for(j = 0; j < numlights; j++) {
			dtolight.x = ship[i].p.x - light[j].p.x;
			dtolight.y = ship[i].p.y - light[j].p.y;
			if(dtolight.x < light[j].lux && dtolight.y < light[j].lux && dtolight.x > -light[j].lux && dtolight.y > -light[j].lux) { //within light effect bounding box
				disttolight[j] = hypot(dtolight.x, dtolight.y) + 1; //add 1 to avoid div by 0
				if(disttolight[j] <= light[j].lux) { //no sense making shadows if they're too far away to have much effect
					//I hate useing trig in real time, but it's faster than last time
					lightang[j] = (byte)((atan2(light[j].p.y - ship[i].p.y, light[j].p.x - ship[i].p.x)) * RADIAN + 64);
					lightaffectsship[j] = TRUE;
				}
				else lightaffectsship[j] = FALSE;
			}
			else lightaffectsship[j] = FALSE;
		}
		for(sprpos.y = 0; sprpos.y < SPRSZE; sprpos.y++) {
			for(sprpos.x = 0; sprpos.x < SPRSZE; sprpos.x++) {
				pxl.x = sprpos.x + ship[i].p.x - HLFSZE;
				pxl.y = sprpos.y + ship[i].p.y - HLFSZE;
				centered.x = sprpos.x - HLFSZE;
				centered.y = sprpos.y - HLFSZE;
				sprpxl.x = centered.x * Cos[ship[i].r] - centered.y * Sin[ship[i].r] + HLFSZE;
				sprpxl.y = centered.y * Cos[ship[i].r] + centered.x * Sin[ship[i].r] + HLFSZE;
				if(sprpxl.x < SPRSZE && sprpxl.x >= 0 && sprpxl.y < SPRSZE && sprpxl.y >= 0 && sprt[i][sprpxl.x][sprpxl.y] != BGCLR) { //if the pixel is in the sprite and is not transparent (BGCLR)
					c = AMBLIGHT;
					for(j = 0; j < numlights && c < SCRMAXCOLOR; j++) {
						if(lightaffectsship[j]) {
							angfromlight = abs(lightang[j] - angs[sprpos.x][sprpos.y]);
							if(angfromlight > 128) angfromlight = 256 - angfromlight;
							//CHANGE TO 16-BIT! :] I'M PUTTING UP WITH THIS DIRECTX WINDOWS STUFF, SO USE IT
							tmpclr = sprt[i][sprpxl.x][sprpxl.y];
							tmpclr *=	(
											(
												angfromlight -
												(disttolight[j] * SHADOWANG) +
												SHADOWDIST
											) +
											(
												lght[sprpos.x][sprpos.y] +
												SHADOWBLEND
											)
										) *
										(light[j].lux / disttolight[j]) /
										SHADOWRATE;
							if(tmpclr < 0) tmpclr = 0;
							else if(tmpclr > SCRMAXCOLOR - c) c = SCRMAXCOLOR;
							else c += tmpclr;
						}
					}
					pset(pxl.x, pxl.y, c);
				}
			}
		}
	}

	//do special effects (explosions, etc)

	//draw the shield bars
	for(i = 0; i < NUMSHIPS; i++) {
		(i == 0) ? (pxl.y = 0) : (pxl.y = SCRY - 1);
		c = BARCLR;
		for(pxl.x = 0; pxl.x < MAXSHIELDS; pxl.x++) {//do shield bar
			if(pxl.x > ship[i].shield) c = BGCLR;
			pset(pxl.x, pxl.y, c);
		}
	}

}

void DoInput(void) {
	//NOTE: dtime proportionalizes these to the unpredictable time delta between checking for input
	// this means that there will be no change between different machine speeds (hopefully)
	// the rounderr check rounding off errors with the ship[].r
	static float rounderr = 0;

	if(KEY_DOWN(CTRL_FORWARD)) {
		ship[SHIP_PLAYER].inertia.x -= Sin[ship[SHIP_PLAYER].r] * ACCEL * dtime;
		ship[SHIP_PLAYER].inertia.y -= Cos[ship[SHIP_PLAYER].r] * ACCEL * dtime;
	}
	if(KEY_DOWN(CTRL_REVERSE)) {
		//do sth?? rotate backwards?
	}
	if(KEY_DOWN(CTRL_LEFT)) {
		float rang = TURNRATE * dtime;

		ship[SHIP_PLAYER].r += (byte)rang;
		rounderr += rang - (byte)rang;
		if(rounderr >= 1) {
			ship[SHIP_PLAYER].r++;
			rounderr--;
		}
		ship[SHIP_PLAYER].r += TURNRATE;
	}
	if(KEY_DOWN(CTRL_RIGHT)) {
		float rang = TURNRATE * dtime;

		ship[SHIP_PLAYER].r -= (byte)rang;
		if(rounderr <= -1) {
			ship[SHIP_PLAYER].r--;
			rounderr++;
		}
		ship[SHIP_PLAYER].r -= TURNRATE;
	}
	if(KEY_DOWN(CTRL_SHOOT1) || KEY_DOWN(CTRL_SHOOT2)) {
		AddShot(SHIP_PLAYER);
	}
}

void DoInput2(void) {
	//NOTE: dtime proportionalizes these to the unpredictable time delta between checking for input
	// this means that there will be no change between different speeds
	// the rounderr check rounding off errors with the ship[].r
	static float rounderr = 0;

	if(KEY_DOWN(CTRL2_FORWARD)) {
		ship[SHIP_CPU].inertia.x -= Sin[ship[SHIP_CPU].r] * ACCEL * dtime;
		ship[SHIP_CPU].inertia.y -= Cos[ship[SHIP_CPU].r] * ACCEL * dtime;
	}
	if(KEY_DOWN(CTRL2_REVERSE)) {
		//do sth?? rotate backwards?
	}
	if(KEY_DOWN(CTRL2_LEFT)) {
		float rang = TURNRATE * dtime;

		ship[SHIP_CPU].r += (byte)rang;
		rounderr += rang - (byte)rang;
		if(rounderr >= 1) {
			ship[SHIP_CPU].r++;
			rounderr--;
		}
		ship[SHIP_CPU].r += TURNRATE;
	}
	if(KEY_DOWN(CTRL2_RIGHT)) {
		float rang = TURNRATE * dtime;

		ship[SHIP_CPU].r -= (byte)rang;
		if(rounderr <= -1) {
			ship[SHIP_CPU].r--;
			rounderr++;
		}
		ship[SHIP_CPU].r -= TURNRATE;
	}
	if(KEY_DOWN(CTRL2_SHOOT1) || KEY_DOWN(CTRL2_SHOOT2)) {
		AddShot(SHIP_CPU);
	}
}

void DoAI(void) {
	vec2_it target;
	static int aimerror = 0;
	int disttoenemy, angtoenemy;

	disttoenemy = hypot(ship[SHIP_PLAYER].p.x - ship[SHIP_CPU].p.x, ship[SHIP_PLAYER].p.y - ship[SHIP_CPU].p.y);
	//the ai ship aims where it thinks the player will be when it's shot gets there
	target.x = ship[SHIP_PLAYER].p.x - Sin[ship[SHIP_PLAYER].r] * ship[SHIP_PLAYER].inertia.x * (disttoenemy / WEPSPEED) - ship[1].inertia.x;
	target.y = ship[SHIP_PLAYER].p.y + Cos[ship[SHIP_PLAYER].r] * ship[SHIP_PLAYER].inertia.y * (disttoenemy / WEPSPEED) - ship[1].inertia.y;
	disttoenemy = hypot(target.x - ship[SHIP_CPU].p.x, target.y - ship[SHIP_CPU].p.y);
	//this next line is ugly
	//angtoenemy = (int)(-atan2(ship[SHIP_PLAYER].p.y - ship[SHIP_CPU].p.y, ship[SHIP_PLAYER].p.x - ship[SHIP_CPU].p.x) * RADIAN + 192) % 256;
	angtoenemy = (int)(-atan2(target.y - ship[SHIP_CPU].p.y, target.x - ship[SHIP_CPU].p.x) * RADIAN + 192) & 255;
	angtoenemy = ship[SHIP_CPU].r - angtoenemy + aimerror;
	if(angtoenemy > 128) angtoenemy = 256 - angtoenemy;
	else if(angtoenemy < -128) angtoenemy = -256 - angtoenemy;

	if(angtoenemy < -AI_TURN_TOLERANCE) ship[SHIP_CPU].r += TURNRATE * dtime;
	else if(angtoenemy > AI_TURN_TOLERANCE) ship[SHIP_CPU].r -= TURNRATE * dtime;
	else {
		if(disttoenemy > AI_TOOCLOSE) {
			ship[SHIP_CPU].inertia.x -= Sin[ship[SHIP_CPU].r] * ACCEL * dtime;
			ship[SHIP_CPU].inertia.y -= Cos[ship[SHIP_CPU].r] * ACCEL * dtime;
		}
		else {
			/* try to shoot (like players who hold down the fire key) */
			AddShot(SHIP_CPU);
			/* change the aim error so the thing isn't always shooting off in the same direction */
			aimerror = (rand() % AI_AIMERROR) - AI_HLFAIMERROR;
		}
	}
}

/* these were used before when the ships looped around the screen, but that was too confusing */

/*void WrapToScreen(int *x, int *y, float *inertiax, float *inertiay) {
	if(*x < LBOUND) {
		*x = LBOUND;
		*inertiax = -(*inertiax) * BOUNCERATIO;
	}
	else if(*x >= RBOUND) {
		*x = RBOUND - 1;
		*inertiax = -(*inertiax) * BOUNCERATIO;
	}
	if(*y < TBOUND) {
		*y = TBOUND;
		*inertiay = -(*inertiay) * BOUNCERATIO;
	}
	else if(*y >= BBOUND) {
		*y = BBOUND - 1;
		*inertiay = -(*inertiay) * BOUNCERATIO;
	}
}

void LoopToScreen(int *x, int *y) {
	if(*x < LBOUND) {
		*x += HZONE;
	}
	else if(*x >= RBOUND) {
		*x -= HZONE;
	}
	if(*y < TBOUND) {
		*y += VZONE;
	}
	else if(*y >= BBOUND) {
		*y -= VZONE;
	}
}*/
